/*
 * Author: Heinz Mauelshagen, Germany
 *
 * March,October,November 1997
 * January,May 1998
 *
 * LVM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * LVM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GNU CC; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA. 
 *
 */

/*
 * Changelog
 *
 *    10/08/1997 - added initialization of logical volume (1st KB zero)
 *    11/08/1997 - added message if default stripe size used
 *    11/09/1997 - added lvmtab handling
 *    04/29/1998 - changed to new lv_create_node()
 *    05/16/1998 - added lvmtab checking
 *    05/26/1998 - added message in case of low space
 *
 */

/*
 * TODO
 *
 * allow striped volume creation on multiple stripe count volumes
 * (lv_setup_for_create() and lv_setup_for_extend concerned())
 *
 */

/*
 * Changelog
 *
 *    01/18/1998 - added option z to give possibility to disallow zeroing 
 *                 of newly created logical volumes
 *
 */

#include <lvm_user.h>

#ifdef DEBUG
int opt_d = 0;
#endif

int main ( int argc, char **argv) {
   int c = 0;
   int i = 0;
   int l = 0;
   int lv_handle = -1;
   int pa = 0;
   int opt_A = 1;
   int opt_C = 0;
   int opt_i = 0;
   int opt_I = 0;
   int opt_l = 0;
   int opt_L = 0;
   int opt_n = 0;
   int opt_p = 0;
   int opt_v = 0;
   int opt_Z = 1;
   int ret = 0;
   uint size = 0;
   int size_rest = 0;
   uint allocation = 0;
   uint mirrors = 0;
   uint permission = LV_READ | LV_WRITE;
   uint stripes = 1;
   uint stripesize = 0;
   ulong pe_free = 0;
   char buf[2*BLOCK_SIZE];
#ifdef DEBUG
   char *options = "A:C:dh?i:I:l:L:n:p:vZ:";
#else
   char *options = "A:C:h?i:I:l:L:n:p:vZ:";
#endif
   char *cmd = NULL;
   char *dummy = NULL;
   char *vg_name = NULL;
   char **pv_allowed = NULL;
   char lv_name[NAME_LEN] = { 0, };
   char lv_path[NAME_LEN] = { 0, };
   pv_t *pv = NULL;
   vg_t *vg = NULL;
   struct stat stat_buf;

   cmd = basename ( argv[0]);

   SUSER_CHECK;
   LVMTAB_CHECK;

   while ( ( c = getopt ( argc, argv, options)) != EOF) {
      switch ( c) {
         case 'A':
            if ( opt_A > 1) {
               fprintf ( stderr, "%s -- A option yet given\n\n", cmd);
               return 1;
            }
            if ( strcmp ( optarg, "y") == 0);
            else if ( strcmp ( optarg, "n") == 0) opt_A = 0;
            else {
               fprintf ( stderr, "%s -- invalid option argument %s\n\n",
                                 cmd, optarg);
               return 1;
            }
            break;

         case 'C':
            if ( opt_C > 0) {
               fprintf ( stderr, "%s -- C option yet given\n\n", cmd);
               return 1;
            }
            opt_C++;
            if ( strcmp ( optarg, "y") == 0) { allocation |= LV_CONTIGUOUS;
                                               break;}
            if ( strcmp ( optarg, "n") == 0) break;
            fprintf ( stderr, "%s -- C option argument \"%s\" invalid\n\n",
                      cmd, optarg);
            return 1;
            break;

#ifdef DEBUG
         case 'd':
            if ( opt_d > 0) {
               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
               return 1;
            }
            opt_d++;
            break;
#endif

         case 'h':
         case '?':
            printf ( "\n%s\n\n%s -- Logical volume create\n\n"
                     "Synopsis:\n"
                     "---------\n\n"
                     "%s\n"
                     "\t[-A y/n]\n"
                     "\t[-C y/n]\n"
#ifdef DEBUG
                     "\t[-d]\n"
#endif
                     "\t[-h/?]\n"
                     "\t[-i Stripes [-I StripeSize]]\n"
                     "\t{-l LogicalExtentsNumber |\n"
                     "\t -L LogicalVolumeSize}\n"
                     "\t[-n LogicalVolumeName]\n"
                     "\t[-p r/rw]\n"
                     "\t[-v]\n"
                     "\t[-Z y/n]\n"
                     "\tVolumeGroupName [PhysicalVolumePath...]\n\n",
                     lvm_version, cmd, cmd);
            return 0;
            break;

         case 'i':
            if ( opt_i > 0) {
               fprintf ( stderr, "%s -- i option yet given\n\n", cmd);
               return 1;
            }
            stripes = atoi ( optarg);
            stripesize = LVM_DEFAULT_STRIPE_SIZE;
            opt_i++;
            break;

         case 'I':
            if ( opt_I > 0) {
               fprintf ( stderr, "%s -- I option yet given\n\n", cmd);
               return 1;
            }
            stripesize = atoi ( optarg);
            opt_I++;
            break;

         case 'l':
            if ( opt_L > 0) {
               fprintf ( stderr, "%s -- L option yet given\n\n", cmd);
               return 1;
            }
            if ( opt_l > 0) {
               fprintf ( stderr, "%s -- l option yet given\n\n", cmd);
               return 1;
            }
            size = atoi ( optarg);
            opt_l++;
            break;

         case 'L':
            if ( opt_l > 0) {
               fprintf ( stderr, "%s -- l option yet given\n\n", cmd);
               return 1;
            }
            if ( opt_L > 0) {
               fprintf ( stderr, "%s -- L option yet given\n\n", cmd);
               return 1;
            }
            size = atoi ( optarg);
            opt_L++;
            break;

         case 'n':
            if ( opt_n > 1) {
               fprintf ( stderr, "%s -- n option yet given\n\n", cmd);
               return 1;
            }
            if ( strchr ( optarg, '/') != NULL) {
               fprintf ( stderr, "%s -- invalid option argument %s\n\n",
                                 cmd, optarg);
               return 1;
            }
            strncpy ( lv_name, optarg, sizeof ( lv_name) - 1);
            lv_name[sizeof ( lv_name) - 1] = 0;
            break;

         case 'p':
            if ( opt_p > 0) {
               fprintf ( stderr, "%s -- p option yet given\n\n", cmd);
               return 1;
            }
            opt_p++;
            if ( strcmp ( optarg, "rw") == 0) {
               permission = ( LV_READ | LV_WRITE);
               break;
            }
            if ( strcmp ( optarg, "r") == 0) { permission = LV_READ; break;}
            fprintf ( stderr, "%s -- p option argument \"%s\" invalid\n\n",
                      cmd, optarg);
            return 1;
            break;

         case 'v':
            if ( opt_v > 0) {
               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
               return 1;
            }
            opt_v++;
            break;

         case 'Z':
            if ( opt_Z > 1) {
               fprintf ( stderr, "%s -- Z option yet given\n\n", cmd);
               return 1;
            }
            if ( strcmp ( optarg, "y") == 0) opt_Z = 1;
            else if ( strcmp ( optarg, "n") == 0) opt_Z = 0;
            else {
               fprintf ( stderr, "%s -- invalid option argument %s\n\n",
                                 cmd, optarg);
               return 1;
            }
            break;

         default:
            fprintf ( stderr, "%s -- invalid command line option \"%c\"\n",
                      cmd, c);
            return 1;
      }
   }

   if ( optind < argc && *argv[optind] == '-') {
      fprintf ( stderr, "%s -- invalid command line\n\n", cmd);
      return 1;
   }

   if ( optind == argc) {
      fprintf ( stderr, "%s -- please enter a volume group name\n\n", cmd);
      return 1;
   }
   vg_name = argv[optind];
   optind++;

   if ( strlen ( lv_name) > 0) {
      sprintf ( lv_path, "/dev/%s/%s%c", vg_name, lv_name, 0);
      if ( lv_check_name ( lv_path) < 0) {
         fprintf ( stderr, "%s -- invalid logical volume name %s\n\n",
                           cmd, lv_name);
         return 1;
      } else if ( lstat ( lv_path, &stat_buf) != -1) {
         lvm_filetype_show ( stat_buf.st_mode, cmd, lv_path);
         return 1;
      }
      else if ( lv_check_exist ( lv_path) == TRUE) {
         fprintf ( stderr, "%s -- logical volume %s allready exists\n\n",
                           cmd, lv_path);
         return 1;
      }
   }

   /* physical volumes in command line */
   if ( optind < argc) {
      if ( opt_v > 0) printf ( "%s -- checking physical volume names\n", cmd);
      for ( i = optind; i < argc; i++) {
         if ( pv_check_name ( argv[i]) < 0) {
            fprintf ( stderr, "%s -- %s is an invalid physical "
                              "volume name\n\n", cmd, argv[i]);
            return 1;
         }
      }
      pv_allowed = &argv[optind];
   }

   if ( opt_i) {
      if ( opt_I == 0) printf ( "%s -- INFO: using default stripe "
                                "size %lu KB\n", cmd,  LVM_DEFAULT_STRIPE_SIZE);
      if ( pv_allowed != NULL) {
         fprintf ( stderr, "%s -- no striped logical volumes with physical "
                           "volumes on command line supported\n\n", cmd);
         return 1;
      }
      if ( stripes == 1) {
         printf ( "%s -- redundant option i with value 1 is default\n", cmd);
         if ( opt_I > 0)
            printf ( "%s -- option I with one stripe ignored\n", cmd);
      }
   }

   if ( opt_v > 0) printf ( "%s -- checking stripe count\n", cmd);
   if ( stripes < 1 || stripes > LVM_MAX_STRIPES) {
      fprintf ( stderr, "%s -- invalid number of stripes: %d\n\n",
                cmd, stripes);
      return 1;
   }

   if ( opt_v > 0) printf ( "%s -- checking stripe size\n", cmd);
   if ( stripesize > 0 && lv_check_stripesize ( stripesize) < 0) {
      fprintf ( stderr, "%s -- invalid stripesize %d\n\n",
                cmd, stripesize);
      return 1;
   }
   stripesize *= 2;
   if ( stripes == 1) stripesize = 0;

   if ( opt_v > 0) printf ( "%s -- checking volume group name\n", cmd);
   if ( vg_check_name ( vg_name) < 0) {
      fprintf ( stderr, "%s -- no valid volume group name %s\n\n",
                cmd, vg_name);
      return 1;
   }

   LVM_LOCK ( 0);
   LVM_CHECK_IOP;

   /* does VG exist? */
   if ( opt_v > 0) printf ( "%s -- checking volume group existence\n", cmd);
   if ( lvm_tab_vg_check_exist ( vg_name, NULL) != TRUE) {
      fprintf ( stderr, "%s -- can't create logical volume:"
                        " %s doesn't exists\n\n", cmd, vg_name);
      return 1;
   }

   if ( opt_v > 0) printf ( "%s -- checking volume group activity\n", cmd);
   if ( vg_check_active ( vg_name) != TRUE) {
      fprintf ( stderr, "%s -- can't create logical volume:"
                        " %s isn't active\n\n", cmd, vg_name);
      return 1;
   }

   if ( opt_v > 0) printf ( "%s -- getting volume group status from "
                            "VGDA in kernel\n", cmd);
   if ( vg_status ( vg_name, &vg) != 0) {
      fprintf ( stderr, "%s -- can't create logical volume"
                        " in inactive volume group %s\n\n", cmd, vg_name);
      return 1;
   }

   if ( opt_v > 0) printf ( "%s -- checking stripe size against "
                            "volume group physical extend size\n", cmd);
   if ( stripesize > vg->pe_size) {
      fprintf ( stderr, "%s -- setting stripe size %d KB to "
                        "physical extend size %lu KB\n",
                        cmd, stripesize / 2, vg->pe_size / 2);
      stripesize = vg->pe_size;
   }

   if ( opt_L > 0 || opt_l > 0) {
      if ( opt_L > 0) size *= 1024 * 2;
      else            size *= vg->pe_size;
      if ( size % vg->pe_size > 0) {
         size += vg->pe_size - size % vg->pe_size;
         printf ( "%s -- rounding up size to physical extend boundary %s\n",
                  cmd, ( dummy = show_size ( size / 2, SHORT)));
         free ( dummy); dummy = NULL;
      }

      size_rest = size % ( stripes * vg->pe_size);
      if ( size_rest != 0) {
         printf ( "%s -- rounding %d KB to stripe boundary size ",
                  cmd, size / 2);
         size = size - size_rest + stripes * vg->pe_size;
         printf ( "%d KB / %lu PE\n", size / 2, size / vg->pe_size);
      }
   } else {
      fprintf ( stderr, "%s -- please give an l or L option\n\n", cmd);
      return 1;
   }

   if ( opt_v > 0) printf ( "%s -- checking volume group free space\n", cmd);
   pe_free = vg->pe_total - vg->pe_allocated;
   if ( pe_free == 0) {
      fprintf ( stderr, "%s -- no free physical extends in %s\n\n",
                cmd, vg_name);
      return 1;
   }
   else if ( size / vg->pe_size > pe_free) {
      fprintf ( stderr, "%s -- only %lu free physical extend", cmd, pe_free);
      if ( pe_free > 1) fprintf ( stderr, "s");
      fprintf ( stderr, " in %s\n\n", vg_name);
      return 1;
   }

   if ( opt_v > 0) printf ( "%s -- checking logical volume maximum size\n",
                            cmd);
   if ( size > LVM_LV_SIZE_MAX( vg)) {
      fprintf ( stderr, "%s -- size %d is to large for the VGDA in kernel\n\n",
                cmd, size / 2);
      return 1;
   }


   if ( opt_v > 0) printf ( "%s -- checking stripe count against "
                            "physical volume count\n", cmd);
   if ( stripes > vg->pv_cur) {
      fprintf ( stderr, "%s -- too many stripes for %s with %lu "
                        "physical volume",
                cmd, vg_name, vg->pv_cur);
      if ( vg->pv_cur > 1) fprintf ( stderr, "s");
      fprintf ( stderr, "\n\n");
      return 1;
   }

   if ( opt_v > 0) printf ( "%s -- reading volume group data of %s\n",
                            cmd, vg_name);
   if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) != 0) {
      fprintf ( stderr, "%s -- ERROR %d: couldn't get volume "
                        "group data of %s\n\n",
                        cmd, ret, vg_name);
      return 1;
   }

   if ( pv_allowed != NULL) {
      for ( pa = 0; pv_allowed[pa] != NULL; pa++) {
         if ( opt_v > 0) printf ( "%s -- checking physical volume %s against "
                                  "volume group\n", pv_allowed[pa], cmd);
         if ( pv_check_in_vg ( vg, pv_allowed[pa]) == FALSE) {
            fprintf ( stderr, "%s -- %s doesn't belong to volume group %s\n\n",
                              cmd, pv_allowed[pa], vg_name);
            return 1;
         }
         if ( pv_read ( pv_allowed[pa], &pv, NULL) < 0) {
            fprintf ( stderr, "%s -- couldn't read physical volume %s\n\n",
                              cmd, pv_allowed[pa]);
            return 1;
         }
         if ( pv->pe_total - pv->pe_allocated == 0)
            printf ( "%s -- no space on physical volume %s\n",
                     cmd, pv_allowed[pa]);
      }
   }

   if ( opt_v > 0) printf ( "%s -- checking for maximum logical volume count\n",
                            cmd);
   if ( vg->lv_cur == vg->lv_max) {
      fprintf ( stderr, "%s -- maximum number of logical volumes "
                        "reached in %s\n\n",
                        cmd, vg_name);
      return 1;
   }

   /* setup logical volume */
   if ( opt_v > 0) printf ( "%s -- setting up logical volume\n", cmd);
   if ( ( ret = lv_setup_for_create ( vg_name, &vg, lv_name, &l, size,
                                      stripes, stripesize, mirrors,
                                      allocation, permission,
                                      pv_allowed)) != 0) {
      if ( ret == -LVM_ESIZE) {
         if ( pv_allowed != NULL)
            fprintf ( stderr, "%s -- not enough allocatable/free physical "
                              "extends on the given physical volumes\n", cmd);
         else
            fprintf ( stderr, "%s -- not enough allocatable/free physical "
                              "volume space in %s\n", cmd, vg_name);
         fprintf ( stderr, "%s -- please check, if physical volumes are "
                           "allocatable\n\n", cmd);
         return 1;
      } else {
         fprintf ( stderr, "%s -- ERROR %d: creating %s in %s\n\n", 
                   cmd, ret, lv_name, vg_name);
         return 1;
      }
   }


   lvm_dont_interrupt ( 0);

   /* create it in kernel */
   if ( opt_v > 0) printf ( "%s -- creating logical volume VGDA "
                            "in kernel\n", cmd);
   if ( ( ret = lv_create ( vg, vg->lv[l], lv_name)) < 0) {
      fprintf ( stderr, "%s -- ERROR %d creating VGDA for %s in kernel\n\n", 
                cmd, ret, lv_name);
      return 1;
   }

   /* store it on disks */
   if ( opt_v > 0) printf ( "%s -- storing logical volume VGDA on disk(s)\n",
                            cmd);
   if ( ( ret = vg_write_with_pv_and_lv ( vg)) != 0) {
      fprintf ( stderr, "%s -- ERROR %d: storing volume group data of %s"
                        " on disks\n\n", 
                cmd, ret, vg_name);
      if ( ( ret = lv_remove ( vg, vg->lv[l], lv_name)) != 0)
         fprintf ( stderr, "%s -- ERROR %d removing VGDA for %s "
                           "from kernel\n\n", cmd, ret, lv_name);
      return 1;
   }

   if ( opt_v > 0) printf ( "%s -- creating device special file\n", cmd);
   if ( ( ret = lv_create_node ( lv_name,
                                 vg->vg_number,
                                 vg->lv[l]->lv_number + 1)) < 0) {
      if ( ret == -LVM_ELV_CREATE_MODE_MKNOD) {
         fprintf ( stderr, "%s -- logical volume node %s allready existent\n\n",
                   cmd, lv_name);
      } else {
         fprintf ( stderr, "%s -- ERROR %d creating logical volume node %s\n\n",
                   cmd, ret, lv_name);
      }
      return 1;
   }

   if ( opt_Z > 0) {
      if ( opt_v > 0) printf ( "%s -- opening logical volume %s for init\n",
                               cmd, lv_name);
      if ( ( lv_handle = open ( lv_name, O_WRONLY)) == -1) {
         fprintf ( stderr, "%s -- ERROR %d opening logical volume %s\n\n",
                   cmd, errno, lv_name);
         return 1;
      }
   
      memset ( buf, 0, 2 * BLOCK_SIZE);
      if ( opt_v > 0) printf ( "%s -- initializing logical volume %s\n",
                               cmd, lv_name);
      if ( write ( lv_handle, buf, 2 * BLOCK_SIZE) != 2 * BLOCK_SIZE) {
         close ( lv_handle);
         fprintf ( stderr, "%s -- ERROR %d initializing logical "
                           "volume %s\n\n", cmd, errno, lv_name);
         return 1;
      }
      fsync ( lv_handle);
      close ( lv_handle);
   }

   if ( opt_v > 0) printf ( "%s -- changing lvmtab\n", cmd);
   if ( vg_cfgbackup ( vg_name, LVMTAB_DIR, cmd, opt_v, vg) == 0 &&
        opt_A > 0) {
      printf ( "%s -- doing automatic backup of %s\n", cmd, vg_name);
      vg_cfgbackup ( vg_name, VG_BACKUP_DIR, cmd, opt_v, vg);
   }

   lvm_interrupt ();
   LVM_UNLOCK ( 0);

   printf ( "%s -- logical volume %s successfully created\n\n",
             cmd, lv_name);

   return 0;
}
